'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
const OPT_REQUIRED = exports.OPT_REQUIRED = 2;
const OPT_OPTIONAL = exports.OPT_OPTIONAL = 4;
const OPT_ARRAY = exports.OPT_ARRAY = 8;
const OPT_EMPTY = exports.OPT_EMPTY = 16;

class Option {

  /**
   * @param {String} name Name for this Option
   * @param {String} shortcut One-character string to be used as short-hand name
   * @param {Number} options Config options. One of the OPT_* constants.<br>
   *                         Defaults to OPT_EMPTY
   * @param {String} description Descirption to show in help()
   * @param {*} def Default value for this option. Not allowed for OPT_REQUIRED
   */
  constructor(name, shortcut, options, description = '', def = null) {
    if (name === null) {
      throw new Error('Option mame cannot be empty.');
    }

    if (typeof shortcut === 'undefined' || shortcut === '') {
      shortcut = null;
    }

    if (!options) {
      options = OPT_EMPTY;
    }

    if (options < 2 || options > 30) {
      throw new Error(`Invalid options for Option: ${name}`);
    }

    if (this.isEmpty() && this.isArray()) {
      throw new Error('Cannot use OPT_EMPTY together w/ OPT_ARRAY');
    }

    if (this.isEmpty() && this.isRequired()) {
      throw new Error('Cannot use OPT_EMPTY together w/ OPT_REQUIRED');
    }

    if (this.isRequired() && def !== null) {
      throw new Error('Cannot use default value together w/ OPT_REQUIRED');
    }

    this._name = name;
    this._shortcut = shortcut;
    this._options = options;
    this._description = description;

    if (!this.isRequired()) {
      this.value = def;
    } else {
      this._value = null;
    }
  }

  get name() {
    return this._name;
  }

  get shortcut() {
    return this._shortcut;
  }

  get description() {
    return this._description;
  }

  get value() {
    return this._value;
  }

  set value(value) {
    if (this.isEmpty() && !(value === null || typeof value === 'boolean')) {
      throw new Error(`Option value passed for OPT_EMPTY: ${this._name}`);
    }

    if (this.isRequired() && (value === null || typeof value === 'undefined')) {
      throw new Error(`Option value omitted for OPT_REQUIRED: ${this._name}`);
    }

    if (this.isEmpty() && (value === null || typeof value === 'undefined')) {
      value = false;
    }

    if (this.isArray() && !(value instanceof Array)) {
      value = [value];
    }

    this._value = value;
  }

  isRequired() {
    return (this._options & OPT_REQUIRED) === OPT_REQUIRED;
  }

  isOptional() {
    return (this._options & OPT_OPTIONAL) === OPT_OPTIONAL;
  }

  isArray() {
    return (this._options & OPT_ARRAY) === OPT_ARRAY;
  }

  isEmpty() {
    return (this._options & OPT_EMPTY) === OPT_EMPTY;
  }
}
exports.default = Option;